Attribute VB_Name = "modFAYAcq"
Option Explicit

' ****************************************************************************************
' * MODULE D'INTERFACAGE AVEC FAYACQ.DLL v1.0
' *
' * Auteur: Mathieu YARD (FAYA Solutions - http://www.fayas.com)
' *
' * Infos sur le code : my@fayas.com
' * Infos commerciales: gf@fayas.com
' *

' ***************************************
' * Description des fonctions de la DLL *
' ***************************************

' * ---

' *
' * FTW_Init(const HWND hWnd, const F_TRACE_LEVELS trace_lvl = F_TRACE_NEVER)
' *
' * USAGE: Initier le dialogue entre fayacq.dll et votre application.
' *        Il est conseill d'appeler cette fonction une seule fois, au dmarrage
' *                                                             de votre application.
' *        Sans cela l'appel aux fonctions FTW_Acq_* serait impossible.
' *
' * hWnd        in  hWnd de la fentre qui sera parente des fentres produites par fayacq.
' *                 Les fentres de selection de source, setup, acquisition, et d'attente
' *                 scanner sont concernes.
' *
' * trace_lvl   in  Niveau de traces demand. (voir l'numeration F_TRACE_LEVELS)
' *                 Le fichier de log s'appel c:\faya.log
' *
' * RETOUR: Non nul si succs; nul si echec.
' *

' * ---

' *
' * FTW_Open()
' *
' * USAGE: Cette fonction ouvre la source TWAIN par dfaut.
' *   Pour changer la source courante utilisez FTW_SelectDS avant FTW_Open().
' *
' * REMARQUE: un appel russi  FTW_Init est impratif avant de pouvoir utiliser FTW_Open().
' *
' * RETOUR: Non nul si succs; nul si echec.
' *

' * ---

' *
' * FTW_Close()
' *
' * USAGE: Cette fonction referme la source TWAIN ouverte.
' *
' * REMARQUE: Cette fonction est protge, un appel superflu ne provoque pas d'erreur.
' *
' * RETOUR: Non nul si succs; nul si echec.
' *

' * ---

' *
' * FTW_SelectDS()
' *
' * USAGE: Cette fonction affiche une bote de dialogue permettant  l'utilisateur de
' *         choisir la source d'acquisition.
' *
' * REMARQUE: La bote de selection est modale. Cela signifie que la fonction ne rend
' *             pas la main  votre application avant que l'utilisateur valide ou anulle.
' *
' * RETOUR: Nul si succs; Erreur de l'numration F_RC si echec.
' *
' * CODES ERREURS:  F_ERR_BAD_STATUS
' *                 F_ERR_USER_CANCELED
' *                 F_ERR_TWAIN_RCERROR
' *

' * ---

' *
' * FTW_Acq_Init()
' *
' * USAGE: Cette fonction initie un transfert avec la source pralablement ouverte
' *         avec FTW_Open().
' *
' * REMARQUE: Obligatoire avant tout autre appel  une fonction FTW_Acq_*.
' *
' * RETOUR: Nul si succs; Erreur de l'enumeration F_RC si echec.
' *
' * CODES ERREURS:  F_ERR_BAD_STATUS
' *                 F_ERR_NOT_ENOUGH_GLOBAL_MEM
' *                 F_ERR_TWAIN_RCERROR
' *

' * ---

' *
' * FTW_GetMisc(int CAP, FTW_NUM& value, int& hCap)
' *
' * USAGE: Cette fonction sert  poser une question prcise (voir l'enumeration TW_CAP)
' *         au driver TWAIN ouvert a l'aide de FTW_Open().
' *
' * CAP         in  Capacit concerne ( choisir dans l'numeration TW_CAP.)
' *
' * value      out  Valeur retourne par la source twain.
' *
' * hCap     in/out in: NULL = ne retourne pas de container de rsultat
' *                     TRUE = retourne un handle correspondant  un container de type
' *                             valeur unique, enumeration, fourchette ou tableau.
' *                 out: Handle global vers le container. Voir CapabilityTests() pour un
' *                      exemple d'utilisation avance.
' *
' * RETOUR: Non nul si succs; nul si echec.
' *

' * ---

' *
' * FTW_SetMisc(int CAP, FTW_NUM& value, int& hCap)
' *
' * USAGE: Cette fonction sert  dfinir un paramtre prcis (voir l'enumeration TW_CAP)
' *         dans un driver TWAIN ouvert a l'aide de FTW_Open().
' *
' * CAP         in  Capacit concerne ( choisir dans l'enumeration TW_CAP.)
' *
' * value      out  Valeur  attribuer.
' *
' * hCap     in/out in: NULL = ne retourne pas de container de rsultat
' *                     TRUE = retourne un handle correspondant  un container de type
' *                             valeur unique, enumeration, fourchette ou tableau.
' *                 out: Handle global vers le container. Voir CapabilityTests() pour un
' *                      exemple d'utilisation avance.
' *
' * RETOUR: Non nul si succs; nul si echec.
' *

' * ---

' *
' * FTW_Acq_AutoFeed()
' *
' * USAGE: Cette fonction active l'acquisition multiple de document.
' *         (par un chargeur de documents, par exemple).
' *        Elle doit tre appele aprs FTW_Acq_Init() et avant FTW_Acq_Go()
' *
' * RETOUR: Non nul si succs; nul si echec.
' *

' * ---

' *
' * FTW_Acq_Resolution(FTW_NUM& XRes, FTW_NUM& YRes)
' *
' * USAGE: Cette fonction sert  dfinir la rsolution d'acquisition horizontale et verticale.
' *        Elle doit tre appele aprs FTW_Acq_Init() et avant FTW_Acq_Go()
' *
' * REMARQUE: Si la valeur spcifie n'est pas disponible fayacq choisit la valeur approchante
' *             la plus convenable et vous la retourne.
' *
' * XRes    in/out  Rsolution X
' *
' * YRes    in/out  Rsolution Y
' *
' * RETOUR: Non nul si succs; nul si echec.
' *

' * ---

' *
' * FTW_Acq_ImageType(FTW_NUM& PixT, FTW_NUM& BitD)
' *
' * USAGE: Cette fonction sert  dfinir le type de pixel ainsi que le taux d'echantillonnage.
' *        Elle doit tre appele aprs FTW_Acq_Init() et avant FTW_Acq_Go()
' *
' * REMARQUE: Si la valeur spcifie n'est pas disponible fayacq choisit la valeur approchante
' *             la plus convenable et vous la retourne.
' *
' * PixT    in/out  Type de pixel parmi TW_PIXELTYPES
' *
' * BitD    in/out  Nombre de bits par pixel de l'image. (communment 1,4,8 ou 24)
' *
' * RETOUR: Non nul si succs; nul si echec.
' *

' * ---

' *
' * FTW_Acq_Go(const int bShowUI)
' *
' * USAGE: Donne le top dpart au pripherique d'acquisition. (dpart de la numrisation, prise de la photo...)
' *         Des appels concluants aux fonctions suivantes doivent obligatoirement prcder:
' *             - FTW_Init, FTW_Open, FTW_Acq_Init
' *
' *
' * bShowUI  in    FLASE ou TRUE. Montre ou non l'Interface Utilisateur (IU)
' *
' * RETOUR: Non nul si succs; nul si echec.
' *

' * ---

' *
' * FTW_MustWait(int& bCanceled)
' *
' * USAGE: Utilise ventuellement pour boucler en attendant la fin d'une acquisition.
' *        Cette fonction ne doit tre appele qu'une fois par acquisition, meme s'il
' *         s'agit d'une acquisition multiple: elle doit prcder le premier appel 
' *         FTW_Acq_GetImage().
' *
' *         Des appels concluants aux fonctions suivantes doivent obligatoirement prcder:
' *             - FTW_Init, FTW_Open, FTW_Acq_Init, FTW_Acq_Go
' *
' * REMARQUE: L'envoi d'un message Form_KeyUp avec un KeyCode
' *              de type FTW_MSGS = FTW_MSG_XFERREADY par fayaacq
' *              peut dispenser votre application de l'utilisation de cette fonction. (FTW_MustWait)
' *           Vous avez donc deux mthodes pour attendre le pripherique d'acquisition
' *             - Utiliser une boucle qui appelle FTW_MustWait (utilisation de DoEvents conseille)
' *             - Ecrire du code dans Form_KeyUp de la fenetre qui a donn son hWnd au moment d'appeler FTW_Init().
' *
' * bCancelled out  FLASE ou TRUE. TRUE si l'utilisateur a annul l'acquisition.
' *
' * RETOUR: Non nul si l'attente s'impose; nul si le priphrique est prt ou si l'utilisateur a annul. (fin de boucle)
' *

' * ---

' *
' * FTW_Acq_GetImage(HANDLE& hDib, int& count)
' *
' * USAGE: Utilise pour rcuprer la prochaine image disponible dans la file des transferts.
' *         (ou l'image unique dans le cas d'un transfert simple.)
' *
' *         Des appels concluants aux fonctions suivantes doivent obligatoirement prcder:
' *             - FTW_Init, FTW_Open, FTW_Acq_Init, FTW_Acq_Go
' *
' * WARNING: Le priphrique doit tre prt avant d'appeler cette fonction.
' *             voir FTW_MustWait() pour plus de dtails
' *
' * hDib  out   Handle global vers l'image (format DIB). De nombreuses autres fonctions
' *              savent exploiter cette valeur pour retraiter ou sauvegarder l'image.
' * count out   Nombre de transferts restant dans la file
' *              (Les priphriques qui ne savent pas compter renvoient 32768 tant que la file est non-vide)
' *
' * RETOUR: Non nul si succs; nul si echec.
' *

' * ---

' *
' * FTW_BMP_SaveImage(HANDLE& hDib, char szPathFile[MAX_PATH])
' *
' * USAGE: Sauvegarde un DIB en mmoire globale dans un fichier BMP. (non compress)
' *
' * REMARQUE: L'envoi d'un hDib erron peut provoquer des rsultats hasardeux.
' *
' * hDib       in  Handle global de l'image.
' * szPathFile in  Chemin et nom du fichier image. Pensez  ajouter un chr(0)
' *                 la fin de votre chane de caractres. (ex: mavar="c:\pwet.bmp" & chr(0))
' *
' * RETOUR: Non nul si succs; nul si echec.
' *

' * ---

' *
' * FTW_BMP_LoadImage(HANDLE& hDib, char szPathFile[MAX_PATH])
' *
' * USAGE: Charge en mmoire globale un fichier BMP (non compress) sous forme de DIB.
' *
' * hDib      out  Handle global de l'image.
' * szPathFile in  Chemin et nom du fichier image. Pensez  ajouter un chr(0)
' *                 la fin de votre chane de caractres. (ex: mavar="c:\pwet.bmp" & chr(0))
' *
' * RETOUR: Non nul si succs; nul si echec.
' *

' * ---

' *
' * FTW_JPG_SaveImage(HANDLE& hDib, char szPathFile[MAX_PATH], int quality)
' *
' * USAGE: Sauvegarde un DIB en mmoire globale dans un fichier JPG.
' *
' * REMARQUES: - L'envoi d'un hDib erron peut provoquer des rsultats hasardeux.
' *            - La version gratuite de FAYAcq.dll limite la dgradation de qualit  95%
' *                    (elle produit donc des fichiers JPG plus volumineux)
' *
' * hDib       in  Handle global de l'image.
' * szPathFile in  Chemin et nom du fichier image. Pensez  ajouter un chr(0)
' *                 la fin de votre chane de caractres. (ex: mavar="c:\pwet.bmp" & chr(0))
' * quality    in  pourcentage de dgradation de la qualit. (conseill entre 30% et 60%)
' *
' * RETOUR: Non nul si succs; nul si echec.
' *

' * ---

' *
' * FTW_JPG_LoadImage(HANDLE& hDib, char szPathFile[MAX_PATH])
' *
' * USAGE: Charge en mmoire globale un fichier JPG sous forme de DIB.
' *
' * hDib      out  Handle global de l'image.
' * szPathFile in  Chemin et nom du fichier image. Pensez  ajouter un chr(0)
' *                 la fin de votre chane de caractres. (ex: mavar="c:\pwet.bmp" & chr(0))
' *
' * RETOUR: Non nul si succs; nul si echec.
' *

' * ---

' *
' * FTW_Acq_GetImageInfo(int cbSz, TW_IMAGEINFO& twImgNfo)
' *
' * USAGE: Demande d'informations sur la prochaine image dans la file des transferts.
' *
' *         Des appels concluants aux fonctions suivantes doivent obligatoirement prcder:
' *             - FTW_Init, FTW_Open, FTW_Acq_Init, FTW_Acq_Go
' *
' * cbSz     in     Taille de la structure twImgInfo fournie par votre application.
' * twImgNfo out    Structure de type TW_IMAGEINFO destin  recevoir les caractristiques.
' *
' * RETOUR: Non nul si succs; nul si echec.
' *

' * ---

' *
' * FTW_Acq_GetImageLayout(int cbSz, TW_IMAGELAYOUT& twImgLayout)
' *
' * USAGE: Demande d'informations sur la disposition du prochain document dans la file des transferts.
' *
' *         Des appels concluants aux fonctions suivantes doivent obligatoirement prcder:
' *             - FTW_Init, FTW_Open, FTW_Acq_Init, FTW_Acq_Go
' *
' * cbSz     in     Taille de la structure twImgLayout fournie par votre application.
' * twImgNfo out    Structure de type TW_IMAGELAYOUT destine  recevoir les caractristiques.
' *
' * RETOUR: Non nul si succs; nul si echec.
' *

' * ---

' *
' * FTW_Get_DataSource(int cbSz, TW_IDENTITY& twID)
' *
' * USAGE: Demande d'informations sur la source courante. (fabriquant, nom produit, versions...)
' *
' * cbSz     in     Taille de la structure twID fournie par votre application.
' * twID     out    Structure de type TW_IDENTITY destine  recevoir les caractristiques.
' *
' * RETOUR: Non nul si succs; nul si echec.
' *

' * ---

' *
' * FTW_SetClipboard(const HGLOBAL hDib)
' *
' * USAGE: Met un DIB en mmoire (image) dans le presse-papier.
' *
' * hDib      in  Handle global de l'image.
' *
' * RETOUR: Non nul si succs; nul si echec.
' *

' * ---

' *
' * FTW_BMP_Rotate(const HBITMAP hDib, const UINT nbQDT)
' *
' * USAGE: Effectuer une copie d'une image en effectuant une rotation  0, 90, 180 ou 270
' *
' * hDib      in  Handle global vers l'image.
' * nbQDT     in  Nombre de quarts de tour dsirs
' *
' * RETOUR: nul si echec; handle global vers la copie retourne si succs.
' *

' * ---

' *
' * FTW_BMP_SetText(const HBITMAP hDib, int x, int y, int w, int h, char szText[])
' *
' * USAGE: Imprime une tiquette de texte en video inverse sur un DIB en mmoire
' *
' * REMARQUES: - Les coordonnes et x, y sont exprimes en pourcentage.
' *                le point (0,0) se trouve en bas a gauche
' *            - Les dimensions w et h sont exprimes en pourcentage galement.
' *            - Ne fonctionne que dans la version enregistre de FAYAcq.dll.
' *                 Renvoi false dans la version gratuite.
' *
' * hDib      in  Handle global vers l'image.
' * x, y      in  Coordonnes du centre du label texte en pourcentage des dimentions du DIB
' * w, h      in  Largeur et hauteur du label
' * szText    in  Texte  imprimer, n'oubliez pas d'ajouter un chr(0)  la fin de votre chaine. (sinon plantage)
' *
' * RETOUR: true ou false (1 ou 0).
' *
' * ---

' *
' * bool LOK()
' *
' * USAGE: Determine si votre license est accepte par la DLL.
' *         N'utilisez pas cette fonction avant l'appel  FTW_Init.
' *
' * REMARQUE: La version gratuite renvoi false!
' *
' * RETOUR: true ou false (1 ou 0).
' *
' * ---

' *
' * Fonctions qui ne sont plus disponibles par rapport aux versions prcedentes
' *
' * FTW_SaveImageDlg(HANDLE& hDib, HWND hWnd)    - Boite de dialogue enregistrer sous
' * FTW_PaintDC(HDC hDc, HANDLE hDib)            - Habille un DC (device context) avec l'image
' * FTW_Draw(HBITMAP hDib)                       - Affiche une fenetre avec l'image (hDib)
' *

' ******************************************
' * Description des fonctions de ce module *
' ******************************************

'  Voir les commentaires des fonctions


' ===== KERNEL ============================================================================

Public Const MAX_PATH = 260

'API error function
Private Declare Function GetLastError Lib "kernel32" () As Long

'API memory functions
Private Declare Function GlobalAlloc Lib "kernel32" (ByVal wFlags As Long, ByVal dwBytes As Long) As Long
Public Declare Function GlobalFree Lib "kernel32" (ByVal hMem As Long) As Long
Private Declare Function GlobalLock Lib "kernel32" (ByVal hMem As Long) As Long
Private Declare Function GlobalUnlock Lib "kernel32" (ByVal hMem As Long) As Long

Private Declare Sub CpyMemValAdrFromRefAdr Lib "kernel32" Alias "RtlMoveMemory" (ByVal hpvDest As Any, hpvSource As Any, ByVal cbCopy As Long)
Private Declare Sub CpyMemRefAdrFromValAdr Lib "kernel32" Alias "RtlMoveMemory" (hpvDest As Any, ByVal hpvSource As Any, ByVal cbCopy As Long)

Private Declare Sub RtlMoveMem Lib "kernel32" Alias "RtlMoveMemory" (hpvDest As Any, hpvSource As Any, ByVal cbCopy As Long)

'constants for API memory functions
Private Const GMEM_MOVEABLE = &H2
Private Const GMEM_ZEROINIT = &H40
Private Const GHND = (GMEM_MOVEABLE Or GMEM_ZEROINIT)


' =========================================================================================
' FAYA - Constantes

Public Enum F_TRACE_LEVELS      ' Niveaux de prcision du rapport. (c:\fayatwain.log)
    F_TRACE_NEVER = 0
    F_TRACE_ERROR = 1
    F_TRACE_HIGH_DEBUG = 2
    F_TRACE_LOW_DEBUG = 3       ' }
    F_TRACE_MAX = 4             ' } Rservs (usage interne)
End Enum


' =========================================================================================
' TWAIN - Structures

Public Type TW_IMAGEINFO
    XResolution  As Long         '/* Resolution in the horizontal              */ TW_FIX32
    YResolution  As Long         '/* Resolution in the vertical                */ TW_FIX32
    ImageWidth   As Long         '/* Columns in the image, -1 if unknown by DS */ TW_INT32
    ImageLength As Long          '/* Rows in the image, -1 if unknown by DS    */ TW_INT32
    SamplesPerPixel As Integer   '/* Number of samples per pixel, 3 for RGB    */ TW_INT16
    BitsPerSample(7) As Integer  '/* Number of bits for each sample            */ TW_INT16
    BitsPerPixel As Integer      '/* Number of bits for each padded pixel      */ TW_INT16
    Planar As Integer            '/* True if Planar, False if chunky           */ TW_BOOL
    PixelType As Integer         '/* How to interp data; photo interp (TWPT_)  */ TW_INT16
    Compression As Integer       '/* How the data is compressed (TWCP_xxxx)    */ TW_UINT16
End Type

Const MAX_TWARRAY = 256

Public Type TW_CAPABILITY
   cap As Integer       'TW_UINT16 /* id of capability to set or get, e.g. CAP_BRIGHTNESS */
   ConType As Integer   'TW_UINT16 /* TWON_ONEVALUE, _RANGE, _ENUMERATION or _ARRAY   */
   hContainer As Long   'TW_HANDLE /* Handle to container of type Dat              */
End Type

Public Type FTW_FIX32
    whole As Integer        '/* maintains the sign */   TW_INT16
    frac  As Integer        '                           TW_UINT16
End Type

Public Type FTW_ONEVALUE
   ItemType As Integer                  '                                           TW_UINT16
   value    As Variant
End Type

Public Type FTW_ARRAY
   ItemType  As Integer                  '                                           TW_UINT16
   NumItems  As Long                     '/* How many items in ItemList           */ TW_UINT32
   ItemList(MAX_TWARRAY - 1) As Variant  '/* Array of ItemType values starts here */ TW_UINT8
End Type

Public Type FTW_ENUMERATION
   ItemType As Integer              '                                                  TW_UINT16
   NumItems As Long                 ' /* How many items in ItemList                 */ TW_UINT32
   CurrentIndex As Long             ' /* Current value is in ItemList[CurrentIndex] */ TW_UINT32
   DefaultIndex As Long             ' /* Powerup value is in ItemList[DefaultIndex] */ TW_UINT32
   ItemList(MAX_TWARRAY - 1) As Variant '                                  TW_UINT8
End Type

Public Type FTW_RANGE
   ItemType As Integer  ' TW_UINT16
   MinValue As Long     ' TW_UINT32       /* Starting value in the range.           */
   MaxValue As Long     ' TW_UINT32       /* Final value in the range.              */
   StepSize As Long     ' TW_UINT32       /* Increment from MinValue to MaxValue.   */
   DefaultValue As Long ' TW_UINT32       /* Power-up value.                        */
   CurrentValue As Long ' TW_UINT32       /* The value that is currently in effect. */
End Type

Public Type TW_FRAME
    Left     As Long 'TW_FIX32
    Top      As Long 'TW_FIX32
    Right    As Long 'TW_FIX32
    Bottom   As Long 'TW_FIX32
End Type

Public Type TW_IMAGELAYOUT
    Frame           As TW_FRAME  '/* Frame coords within larger document */ TW_FRAME
    DocumentNumber  As Long      '/*                                     */ TW_UINT32
    PageNumber      As Long      '/* Reset when you go to next document  */ TW_UINT32
    FrameNumber     As Long      '/* Reset when you go to next page      */ TW_UINT32
End Type

' FTW - Messages retourns par ezscan (Evenement KeyUp)
Public Enum FTW_MSGS
    FTW_MSG_NULL = &H10
    FTW_MSG_XFERREADY = &H1
    FTW_MSG_CLOSEDS_REQ = &H2
    FTW_MSG_CLOSEDS_OK = &H3
End Enum

Public Declare Function FTW_Init Lib "fayacq.dll" (ByVal hWnd As Long, Optional ByVal iDebugLvl As Long) As Long
Public Declare Function FTW_Open Lib "fayacq.dll" () As Long
Public Declare Function FTW_Close Lib "fayacq.dll" () As Long

Public Declare Function FTW_MustWait Lib "fayacq.dll" (bCanceled As Long) As Long

Public Declare Function FTW_GetMisc Lib "fayacq.dll" (ByVal cap As Long, ByRef value As Single, ByRef hCap As Long) As Long
Public Declare Function FTW_SetMisc Lib "fayacq.dll" (ByVal cap As Long, ByRef value As Single, ByRef hCap As Long) As Long

Public Declare Function FTW_Acq_Init Lib "fayacq.dll" () As Long
Public Declare Function FTW_Acq_AutoFeed Lib "fayacq.dll" () As Long
Public Declare Function FTW_Acq_Resolution Lib "fayacq.dll" (ByRef XRes As Single, ByRef YRes As Single) As Long
Public Declare Function FTW_Acq_ImageType Lib "fayacq.dll" (ByRef PixT As Single, ByRef BitD As Single) As Long
Public Declare Function FTW_Acq_Go Lib "fayacq.dll" (ByVal bShowUI As Long, Optional ByVal hWnd As Long) As Long
Public Declare Function FTW_Acq_GetImageInfo Lib "fayacq.dll" (ByVal cbsz As Long, ByRef twInfo As Any) As Long
Public Declare Function FTW_Acq_GetImageLayout Lib "fayacq.dll" (ByVal cbsz As Long, ByRef twLayout As Any) As Long
Public Declare Function FTW_Acq_GetImage Lib "fayacq.dll" (ByRef hDib As Long, ByRef cnt As Long) As Long

Public Declare Function FTW_BMP_SaveImage Lib "fayacq.dll" (ByVal hDib As Long, ByVal szPathFile As String) As Long
Public Declare Function FTW_BMP_LoadImage Lib "fayacq.dll" (ByRef hDib As Long, ByVal szPathFile As String) As Long

' Notez que le paramtre quality de FTW_JPG_SaveImage est un pourcentage... (0=compression maximale; 100=qualit optimale)
Public Declare Function FTW_JPG_SaveImage Lib "fayacq.dll" (ByVal hDib As Long, ByVal szPathFile As String, ByVal quality As Long) As Long
Public Declare Function FTW_JPG_LoadImage Lib "fayacq.dll" (ByRef hDib As Long, ByVal szPathFile As String) As Long

Public Declare Function FTW_GetClipboard Lib "fayacq.dll" () As Long
Public Declare Function FTW_SetClipboard Lib "fayacq.dll" (ByVal hDib As Long) As Long

Public Declare Function FTW_SelectDS Lib "fayacq.dll" () As Long

Public Declare Function FTW_BMP_Rotate Lib "fayacq.dll" (ByVal hDib As Long, ByVal nbQDT As Long) As Long

Public Declare Function FTW_BMP_SetText Lib "fayacq.dll" (ByVal hDib As Long, ByVal x As Long, ByVal y As Long, ByVal w As Long, ByVal h As Long, ByVal szText As String) As Long


'Public Declare Sub FTW_Debug Lib "fayacq.dll" ()
'Public Declare Sub FTW_Draw Lib "fayacq.dll" (ByVal hDib As Long)
'Public Declare Function FTW_PaintDC Lib "fayacq.dll" (ByVal hDc As Long, ByVal hDib As Long) As Long
'Public Declare Function FTW_SaveImageDlg Lib "fayacq.dll" (ByRef hDib As Long, ByVal hWnd As Long) As Long

' DATA GROUPS
' ***********

Public Const DG_CONTROL = &H1
Public Const DG_IMAGE = &H2
Public Const DG_AUDIO = &H4      ' depuis v1.8

' DATA ARGUMENT TYPE
' ******************

Public Const DAT_NULL = &H0                 '/* No data or structure. */
Public Const DAT_CUSTOMBASE = &H8000        '/* Base of custom DATs.  */

'/* Data Argument Types for the DG_CONTROL Data Group. */
Public Const DAT_CAPABILITY = &H1           '/* TW_CAPABILITY                        */
Public Const DAT_EVENT = &H2                '/* TW_EVENT                             */
Public Const DAT_IDENTITY = &H3             '/* TW_IDENTITY                          */
Public Const DAT_PARENT = &H4               '/* TW_HANDLE, application win handle in Windows */
Public Const DAT_PENDINGXFERS = &H5         '/* TW_PENDINGXFERS                      */
Public Const DAT_SETUPMEMXFER = &H6         '/* TW_SETUPMEMXFER                      */
Public Const DAT_SETUPFILEXFER = &H7        '/* TW_SETUPFILEXFER                     */
Public Const DAT_STATUS = &H8               '/* TW_STATUS                            */
Public Const DAT_USERINTERFACE = &H9        '/* TW_USERINTERFACE                     */
Public Const DAT_XFERGROUP = &HA            '/* TW_UINT32                            */

'/*  SDH - 03/21/95 - TWUNK                                         */
'/*  Additional message required for thunker to request the special */
'/*  identity information.                                          */

Public Const DAT_TWUNKIDENTITY = &HB        '/* TW_TWUNKIDENTITY                     */
Public Const DAT_CUSTOMDSDATA = &HC         '/* TW_CUSTOMDSDATA.                     */
'/* Added 1.8 */
Public Const DAT_DEVICEEVENT = &HD          '/* TW_DEVICEEVENT                       */
Public Const DAT_FILESYSTEM = &HE           '/* TW_FILESYSTEM                        */
Public Const DAT_PASSTHRU = &HF             '/* TW_PASSTHRU                          */

'/* Data Argument Types for the DG_IMAGE Data Group. */
Public Const DAT_IMAGEINFO = &H101          '/* TW_IMAGEINFO                         */
Public Const DAT_IMAGELAYOUT = &H102        '/* TW_IMAGELAYOUT                       */
Public Const DAT_IMAGEMEMXFER = &H103       '/* TW_IMAGEMEMXFER                      */
Public Const DAT_IMAGENATIVEXFER = &H104    '/* TW_UINT32 loword is hDIB, PICHandle  */
Public Const DAT_IMAGEFILEXFER = &H105      '/* Null data                            */
Public Const DAT_CIECOLOR = &H106           '/* TW_CIECOLOR                          */
Public Const DAT_GRAYRESPONSE = &H107       '/* TW_GRAYRESPONSE                      */
Public Const DAT_RGBRESPONSE = &H108        '/* TW_RGBRESPONSE                       */
Public Const DAT_JPEGCOMPRESSION = &H109    '/* TW_JPEGCOMPRESSION                   */
Public Const DAT_PALETTE8 = &H10A           '/* TW_PALETTE8                          */
Public Const DAT_EXTIMAGEINFO = &H10B       '/* TW_EXTIMAGEINFO -- for 1.7 Spec.     */

'/* Data Argument Types for the DG_AUDIO Data Group. */

Public Const DAT_AUDIOFILEXFER = &H201      '/* Null data                            */
Public Const DAT_AUDIOINFO = &H202          '/* TW_AUDIOINFO                         */
Public Const DAT_AUDIONATIVEXFER = &H203    '/* TW_UINT32 handle to WAV, (AIFF Mac)  */



' All message constants are unique.
' * Messages are grouped according to which DATs they are used with.*/

Public Const MSG_NULL = &H0            ' Used in TW_EVENT structure               */
Public Const MSG_CUSTOMBASE = &H8000   ' Base of custom messages                  */

' Generic messages may be used with any of several DATs.                   */

Public Const MSG_GET = &H1             ' Get one or more values                   */
Public Const MSG_GETCURRENT = &H2      ' Get current value                        */
Public Const MSG_GETDEFAULT = &H3      ' Get default (e.g. power up) value        */
Public Const MSG_GETFIRST = &H4        ' Get first of a series of items, e.g. DSs */
Public Const MSG_GETNEXT = &H5         ' Iterate through a series of items.       */
Public Const MSG_SET = &H6             ' Set one or more values                   */
Public Const MSG_RESET = &H7           ' Set current value to default value       */
Public Const MSG_QUERYSUPPORT = &H8    ' Get supported operations on the cap.     */

' Messages used with DAT_NULL                                              */

Public Const MSG_XFERREADY = &H101     ' The data source has data ready           */
Public Const MSG_CLOSEDSREQ = &H102    ' Request for Application. to close DS             */
Public Const MSG_CLOSEDSOK = &H103     ' Tell the Application. to save the state.         */
' Added 1.8 */
Public Const MSG_DEVICEEVENT = &H104   ' Some event has taken place               */

' Messages used with a pointer to a DAT_STATUS structure                   */
Public Const MSG_CHECKSTATUS = &H201   ' Get status information                   */

' Messages used with a pointer to DAT_PARENT data                          */
Public Const MSG_OPENDSM = &H301       ' Open the DSM                             */
Public Const MSG_CLOSEDSM = &H302      ' Close the DSM                            */

' Messages used with a pointer to a DAT_IDENTITY structure                 */
Public Const MSG_OPENDS = &H401        ' Open a data source                       */
Public Const MSG_CLOSEDS = &H402       ' Close a data source                      */
Public Const MSG_USERSELECT = &H403    ' Put up a dialog of all DS                */

' Messages used with a pointer to a DAT_USERINTERFACE structure            */

Public Const MSG_DISABLEDS = &H501          ' Disable data transfer in the DS          */
Public Const MSG_ENABLEDS = &H502           ' Enable data transfer in the DS           */
Public Const MSG_ENABLEDSUIONLY = &H503     ' Enable for saving DS state only.     */

' Messages used with a pointer to a DAT_EVENT structure                    */
Public Const MSG_PROCESSEVENT = &H601

' Messages used with a pointer to a DAT_PENDINGXFERS structure             */
Public Const MSG_ENDXFER = &H701

' Added 1.8 */
' Messages used with a pointer to a DAT_FILESYSTEM structure               */
Public Const MSG_CHANGEDIRECTORY = &H801
Public Const MSG_CREATEDIRECTORY = &H802
Public Const MSG_DELETE = &H803
Public Const MSG_FORMATMEDIA = &H804
Public Const MSG_GETCLOSE = &H805
Public Const MSG_GETFIRSTFILE = &H806
Public Const MSG_GETINFO = &H807
Public Const MSG_GETNEXTFILE = &H808
Public Const MSG_RENAME = &H809

' Messages used with a pointer to a DAT_PASSTHRU structure                 */
Public Const MSG_PASSTHRU = &H901

Public Const CAP_CUSTOMBASE = &H8000          ' Base of custom capabilities */

Public Enum TW_FILEFORMATS
    TWFF_TIFF = 0           '/* Tagged Image File Format     */
    TWFF_PICT = 1           '/* Macintosh PICT               */
    TWFF_BMP = 2            '/* Windows Bitmap               */
    TWFF_XBM = 3            '/* X-Windows Bitmap             */
    TWFF_JFIF = 4           '/* JPEG File Interchange Format */
    TWFF_FPX = 5            '/* Flash Pix                    */
    TWFF_TIFFMULTI = 6      '/* Multi-page tiff file         */
    TWFF_PNG = 7            '/* Portable Network Graphics    */
    TWFF_SPIFF = 8
    TWFF_EXIF = 9
End Enum

Public Enum TW_CONTAINERS
    TWON_ARRAY = 3          '/* indicates TW_ARRAY container=  */
    TWON_ENUMERATION = 4    '/* indicates TW_ENUMERATION container */
    TWON_ONEVALUE = 5       '/* indicates TW_ONEVALUE container    */
    TWON_RANGE = 6          '/* indicates TW_RANGE container=  */
    TWON_ICONID = 962       '/* res Id of icon used in USERSELECT lbox */
    TWON_DSMID = 461        '/* res Id of the DSM version num resource */
    TWON_DSMCODEID = 63     '/* res Id of the Mac SM Code resource=*/
    TWON_DONTCARE8 = &HFF
    TWON_DONTCARE16 = &HFFFF
    TWON_DONTCARE32 = &HFFFFFFFF
End Enum

Public Enum TW_CHUNKY
     TWPC_CHUNKY = 0
     TWPC_PLANAR = 1
End Enum

'/* ICAP_PIXELFLAVOR values (PF_ means Pixel Flavor) */
Public Enum TW_PIXELFLAVOR
     TWPF_CHOCOLATE = 0   '/* zero pixel represents darkest shade  */
     TWPF_VANILLA = 1     '/* zero pixel represents lightest shade */
End Enum

'/* ICAP_PIXELTYPE values (PT_ means Pixel Type) */
Public Enum TW_PIXELTYPES
     TWPT_BW = 0        '/* Black and White */
     TWPT_GRAY = 1
     TWPT_RGB = 2
     TWPT_PALETTE = 3
     TWPT_CMY = 4
     TWPT_CMYK = 5
     TWPT_YUV = 6
     TWPT_YUVK = 7
     TWPT_CIEXYZ = 8
End Enum

'/* ICAP_SUPPORTEDSIZES values (SS_ means Supported Sizes) */
Public Enum TW_SUPPORTEDSIZES
     TWSS_NONE = 0
     TWSS_A4LETTER = 1
     TWSS_B5LETTER = 2
     TWSS_USLETTER = 3
     TWSS_USLEGAL = 4
' /* Added 1.5 */
     TWSS_A5 = 5
     TWSS_B4 = 6
     TWSS_B6 = 7
     TWSS_B = 8
' /* Added 1.7 */
     TWSS_USLEDGER = 9
     TWSS_USEXECUTIVE = 10
     TWSS_A3 = 11
     TWSS_B3 = 12
     TWSS_A6 = 13
     TWSS_C4 = 14
     TWSS_C5 = 15
     TWSS_C6 = 16
' /* Added 1.8 */
     TWSS_4A0 = 17
     TWSS_2A0 = 18
     TWSS_A0 = 19
     TWSS_A1 = 20
     TWSS_A2 = 21
     TWSS_A4 = TWSS_A4LETTER
     TWSS_A7 = 22
     TWSS_A8 = 23
     TWSS_A9 = 24
     TWSS_A10 = 25
     TWSS_ISOB0 = 26
     TWSS_ISOB1 = 27
     TWSS_ISOB2 = 28
     TWSS_ISOB3 = TWSS_B3
     TWSS_ISOB4 = TWSS_B4
     TWSS_ISOB5 = 29
     TWSS_ISOB6 = TWSS_B6
     TWSS_ISOB7 = 30
     TWSS_ISOB8 = 31
     TWSS_ISOB9 = 32
     TWSS_ISOB10 = 33
     TWSS_JISB0 = 34
     TWSS_JISB1 = 35
     TWSS_JISB2 = 36
     TWSS_JISB3 = 37
     TWSS_JISB4 = 38
     TWSS_JISB5 = TWSS_B5LETTER
     TWSS_JISB6 = 39
     TWSS_JISB7 = 40
     TWSS_JISB8 = 41
     TWSS_JISB9 = 42
     TWSS_JISB10 = 43
     TWSS_C0 = 44
     TWSS_C1 = 45
     TWSS_C2 = 46
     TWSS_C3 = 47
     TWSS_C7 = 48
     TWSS_C8 = 49
     TWSS_C9 = 50
     TWSS_C10 = 51
     TWSS_USSTATEMENT = 52
     TWSS_BUSINESSCARD = 53
End Enum

'/* ICAP_XFERMECH values (SX_ means Setup XFer) */
Public Enum TW_SETUPXFERS
     TWSX_NATIVE = 0
     TWSX_FILE = 1
     TWSX_MEMORY = 2
End Enum

'/* ICAP_UNITS values (UN_ means UNits) */
Public Enum TW_UNITS
     TWUN_INCHES = 0
     TWUN_CENTIMETERS = 1
     TWUN_PICAS = 2
     TWUN_POINTS = 3
     TWUN_TWIPS = 4
     TWUN_PIXELS = 5
End Enum

Public Enum TW_TYPES
    TWTY_INT8 = &H0
    TWTY_INT16 = &H1
    TWTY_INT32 = &H2
    
    TWTY_UINT8 = &H3
    TWTY_UINT16 = &H4
    TWTY_UINT32 = &H5
    
    TWTY_BOOL = &H6
    
    TWTY_FIX32 = &H7
    
    TWTY_FRAME = &H8
    
    TWTY_STR32 = &H9
    TWTY_STR64 = &HA
    TWTY_STR128 = &HB
    TWTY_STR255 = &HC
End Enum

Public Enum TW_CAP
    ' all data sources are REQUIRED to support these caps */
    CAP_XFERCOUNT = &H1

    ' image data sources are REQUIRED to support these caps */
    ICAP_COMPRESSION = &H100
    ICAP_PIXELTYPE = &H101

    ICAP_UNITS = &H102               ' default is TWUN_INCHES */
    ICAP_XFERMECH = &H103

    ' all data sources MAY support these caps */
    CAP_AUTHOR = &H1000
    CAP_CAPTION = &H1001
    CAP_FEEDERENABLED = &H1002
    CAP_FEEDERLOADED = &H1003
    CAP_TIMEDATE = &H1004
    CAP_SUPPORTEDCAPS = &H1005
    CAP_EXTENDEDCAPS = &H1006
    CAP_AUTOFEED = &H1007
    CAP_CLEARPAGE = &H1008
    CAP_FEEDPAGE = &H1009
    CAP_REWINDPAGE = &H100A
    CAP_INDICATORS = &H100B                ' Added 1.1 */
    CAP_SUPPORTEDCAPSEXT = &H100C          ' Added 1.6 */
    CAP_PAPERDETECTABLE = &H100D           ' Added 1.6 */
    CAP_UICONTROLLABLE = &H100E            ' Added 1.6 */
    CAP_DEVICEONLINE = &H100F              ' Added 1.6 */
    CAP_AUTOSCAN = &H1010                  ' Added 1.6 */
    CAP_THUMBNAILSENABLED = &H1011         ' Added 1.7 */
    CAP_DUPLEX = &H1012                    ' Added 1.7 */
    CAP_DUPLEXENABLED = &H1013             ' Added 1.7 */
    CAP_ENABLEDSUIONLY = &H1014            ' Added 1.7 */
    CAP_CUSTOMDSDATA = &H1015              ' Added 1.7 */
    CAP_ENDORSER = &H1016                  ' Added 1.7 */
    CAP_JOBCONTROL = &H1017                ' Added 1.7 */
    CAP_ALARMS = &H1018                    ' Added 1.8 */
    CAP_ALARMVOLUME = &H1019               ' Added 1.8 */
    CAP_AUTOMATICCAPTURE = &H101A          ' Added 1.8 */
    CAP_TIMEBEFOREFIRSTCAPTURE = &H101B    ' Added 1.8 */
    CAP_TIMEBETWEENCAPTURES = &H101C       ' Added 1.8 */
    CAP_CLEARBUFFERS = &H101D              ' Added 1.8 */
    CAP_MAXBATCHBUFFERS = &H101E           ' Added 1.8 */
    CAP_DEVICETIMEDATE = &H101F            ' Added 1.8 */
    CAP_POWERSUPPLY = &H1020               ' Added 1.8 */
    CAP_CAMERAPREVIEWUI = &H1021           ' Added 1.8 */
    CAP_DEVICEEVENT = &H1022               ' Added 1.8 */
    CAP_PAGEMULTIPLEACQUIRE = &H1023       ' Added 1.8 */
    CAP_SERIALNUMBER = &H1024              ' Added 1.8 */
    CAP_FILESYSTEM = &H1025                ' Added 1.8 */
    CAP_PRINTER = &H1026                   ' Added 1.8 */
    CAP_PRINTERENABLED = &H1027            ' Added 1.8 */
    CAP_PRINTERINDEX = &H1028              ' Added 1.8 */
    CAP_PRINTERMODE = &H1029               ' Added 1.8 */
    CAP_PRINTERSTRING = &H102A             ' Added 1.8 */
    CAP_PRINTERSUFFIX = &H102B             ' Added 1.8 */
    CAP_LANGUAGE = &H102C                  ' Added 1.8 */
    CAP_FEEDERALIGNMENT = &H102D           ' Added 1.8 */
    CAP_FEEDERORDER = &H102E               ' Added 1.8 */
    CAP_PAPERBINDING = &H102F              ' Added 1.8 */
    CAP_REACQUIREALLOWED = &H1030          ' Added 1.8 */
    CAP_PASSTHRU = &H1031                  ' Added 1.8 */
    CAP_BATTERYMINUTES = &H1032            ' Added 1.8 */
    CAP_BATTERYPERCENTAGE = &H1033         ' Added 1.8 */
    CAP_POWERDOWNTIME = &H1034             ' Added 1.8 */
 
    ' image data sources MAY support these caps */
    ICAP_AUTOBRIGHT = &H1100
    ICAP_BRIGHTNESS = &H1101
    ICAP_CONTRAST = &H1103
    ICAP_CUSTHALFTONE = &H1104
    ICAP_EXPOSURETIME = &H1105
    ICAP_FILTER = &H1106
    ICAP_FLASHUSED = &H1107
    ICAP_GAMMA = &H1108
    ICAP_HALFTONES = &H1109
    ICAP_HIGHLIGHT = &H110A
    ICAP_IMAGEFILEFORMAT = &H110C
    ICAP_LAMPSTATE = &H110D
    ICAP_LIGHTSOURCE = &H110E
    ICAP_ORIENTATION = &H1110
    ICAP_PHYSICALWIDTH = &H1111
    ICAP_PHYSICALHEIGHT = &H1112
    ICAP_SHADOW = &H1113
    ICAP_FRAMES = &H1114
    ICAP_XNATIVERESOLUTION = &H1116
    ICAP_YNATIVERESOLUTION = &H1117
    ICAP_XRESOLUTION = &H1118
    ICAP_YRESOLUTION = &H1119
    ICAP_MAXFRAMES = &H111A
    ICAP_TILES = &H111B
    ICAP_BITORDER = &H111C
    ICAP_CCITTKFACTOR = &H111D
    ICAP_LIGHTPATH = &H111E
    ICAP_PIXELFLAVOR = &H111F
    ICAP_PLANARCHUNKY = &H1120
    ICAP_ROTATION = &H1121
    ICAP_SUPPORTEDSIZES = &H1122
    ICAP_THRESHOLD = &H1123
    ICAP_XSCALING = &H1124
    ICAP_YSCALING = &H1125
    ICAP_BITORDERCODES = &H1126
    ICAP_PIXELFLAVORCODES = &H1127
    ICAP_JPEGPIXELTYPE = &H1128
    ICAP_TIMEFILL = &H112A
    ICAP_BITDEPTH = &H112B
    ICAP_BITDEPTHREDUCTION = &H112C             ' Added 1.5 */
    ICAP_UNDEFINEDIMAGESIZE = &H112D            ' Added 1.6 */
    ICAP_IMAGEDATASET = &H112E                  ' Added 1.7 */
    ICAP_EXTIMAGEINFO = &H112F                  ' Added 1.7 */
    ICAP_MINIMUMHEIGHT = &H1130                 ' Added 1.7 */
    ICAP_MINIMUMWIDTH = &H1131                  ' Added 1.7 */
    ICAP_AUTODISCARDBLANKPAGES = &H1134         ' Added 1.8 */
    ICAP_FLIPROTATION = &H1136                  ' Added 1.8 */
    ICAP_BARCODEDETECTIONENABLED = &H1137       ' Added 1.8 */
    ICAP_SUPPORTEDBARCODETYPES = &H1138         ' Added 1.8 */
    ICAP_BARCODEMAXSEARCHPRIORITIES = &H1139    ' Added 1.8 */
    ICAP_BARCODESEARCHPRIORITIES = &H113A       ' Added 1.8 */
    ICAP_BARCODESEARCHMODE = &H113B             ' Added 1.8 */
    ICAP_BARCODEMAXRETRIES = &H113C             ' Added 1.8 */
    ICAP_BARCODETIMEOUT = &H113D                ' Added 1.8 */
    ICAP_ZOOMFACTOR = &H113E                    ' Added 1.8 */
    ICAP_PATCHCODEDETECTIONENABLED = &H113F     ' Added 1.8 */
    ICAP_SUPPORTEDPATCHCODETYPES = &H1140       ' Added 1.8 */
    ICAP_PATCHCODEMAXSEARCHPRIORITIES = &H1141  ' Added 1.8 */
    ICAP_PATCHCODESEARCHPRIORITIES = &H1142     ' Added 1.8 */
    ICAP_PATCHCODESEARCHMODE = &H1143           ' Added 1.8 */
    ICAP_PATCHCODEMAXRETRIES = &H1144           ' Added 1.8 */
    ICAP_PATCHCODETIMEOUT = &H1145              ' Added 1.8 */
    ICAP_FLASHUSED2 = &H1146                    ' Added 1.8 */
    ICAP_IMAGEFILTER = &H1147                   ' Added 1.8 */
    ICAP_NOISEFILTER = &H1148                   ' Added 1.8 */
    ICAP_OVERSCAN = &H1149                      ' Added 1.8 */
    ICAP_AUTOMATICBORDERDETECTION = &H1150      ' Added 1.8 */
    ICAP_AUTOMATICDESKEW = &H1151               ' Added 1.8 */
    ICAP_AUTOMATICROTATE = &H1152               ' Added 1.8 */

    ' image data sources MAY support these audio caps */
    ACAP_AUDIOFILEFORMAT = &H1201     ' Added 1.8 */
    ACAP_XFERMECH = &H1202            ' Added 1.8 */

End Enum

' ===== LINKING CONSTANTS NAMES WITH VALUES =======================================
Public Function FTW_CapValueName(cap As TW_CAP, index, bForceFull As Boolean) As String
    'Renvoi le nom d'une constante relative aux capacits
        
    Dim strCat As String
    If (IsNumeric(index)) Then
        Select Case cap
            Case TW_CAP.ICAP_XFERMECH
                FTW_CapValueName = GetSetupXferName(CInt(index))
                strCat = "sx"
            Case TW_CAP.ICAP_IMAGEFILEFORMAT
                FTW_CapValueName = GetFileFormatName(CInt(index))
                strCat = "ff"
            Case TW_CAP.ICAP_PIXELFLAVOR
                FTW_CapValueName = GetPixelFlavorName(CInt(index))
                strCat = "pf"
            Case TW_CAP.ICAP_UNITS
                FTW_CapValueName = GetUnitsName(CInt(index))
                strCat = "un"
            Case TW_CAP.ICAP_PIXELTYPE
                FTW_CapValueName = GetPixelTypeName(CInt(index))
                strCat = "pt"
            Case TW_CAP.ICAP_PLANARCHUNKY
                FTW_CapValueName = GetChunkyName(CInt(index))
                strCat = "pc"
            Case TW_CAP.ICAP_SUPPORTEDSIZES
                FTW_CapValueName = GetSupSizeName(CInt(index))
                strCat = "ss"
            Case TW_CAP.CAP_SUPPORTEDCAPS, TW_CAP.CAP_SUPPORTEDCAPSEXT, TW_CAP.CAP_EXTENDEDCAPS
                FTW_CapValueName = GetCapName(CLng(cap))
                strCat = "cap"
        End Select
    End If
    If (bForceFull) And FTW_CapValueName = "" Then
        If strCat <> "" Then
            FTW_CapValueName = strCat & "_" & index
        Else
            FTW_CapValueName = index
        End If
    End If
End Function
Public Function GetCapName(index As TW_CAP) As String
    ' Retourne le nom de l'etiquette d'une capacit donne
    Select Case index
        Case CAP_XFERCOUNT
            GetCapName = "CAP_XFERCOUNT"
        Case ICAP_COMPRESSION
            GetCapName = "ICAP_COMPRESSION"
        Case ICAP_PIXELTYPE
            GetCapName = "ICAP_PIXELTYPE"
        Case ICAP_UNITS
            GetCapName = "ICAP_UNITS"
        Case ICAP_XFERMECH
            GetCapName = "ICAP_XFERMECH"
        Case CAP_AUTHOR
            GetCapName = "CAP_AUTHOR"
        Case CAP_CAPTION
            GetCapName = "CAP_CAPTION"
        Case CAP_FEEDERENABLED
            GetCapName = "CAP_FEEDERENABLED"
        Case CAP_FEEDERLOADED
            GetCapName = "CAP_FEEDERLOADED"
        Case CAP_TIMEDATE
            GetCapName = "CAP_TIMEDATE"
        Case CAP_SUPPORTEDCAPS
            GetCapName = "CAP_SUPPORTEDCAPS"
        Case CAP_EXTENDEDCAPS
            GetCapName = "CAP_EXTENDEDCAPS"
        Case CAP_AUTOFEED
            GetCapName = "CAP_AUTOFEED"
        Case CAP_CLEARPAGE
            GetCapName = "CAP_CLEARPAGE"
        Case CAP_FEEDPAGE
            GetCapName = "CAP_FEEDPAGE"
        Case CAP_REWINDPAGE
            GetCapName = "CAP_REWINDPAGE"
        Case CAP_INDICATORS
            GetCapName = "CAP_INDICATORS"
        Case CAP_SUPPORTEDCAPSEXT
            GetCapName = "CAP_SUPPORTEDCAPSEXT"
        Case CAP_PAPERDETECTABLE
            GetCapName = "CAP_PAPERDETECTABLE"
        Case CAP_UICONTROLLABLE
            GetCapName = "CAP_UICONTROLLABLE"
        Case CAP_DEVICEONLINE
            GetCapName = "CAP_DEVICEONLINE"
        Case CAP_AUTOSCAN
            GetCapName = "CAP_AUTOSCAN"
        Case CAP_THUMBNAILSENABLED
            GetCapName = "CAP_THUMBNAILSENABLED"
        Case CAP_DUPLEX
            GetCapName = "CAP_DUPLEX"
        Case CAP_DUPLEXENABLED
            GetCapName = "CAP_DUPLEXENABLED"
        Case CAP_ENABLEDSUIONLY
            GetCapName = "CAP_ENABLEDSUIONLY"
        Case CAP_CUSTOMDSDATA
            GetCapName = "CAP_CUSTOMDSDATA"
        Case CAP_ENDORSER
            GetCapName = "CAP_ENDORSER"
        Case CAP_JOBCONTROL
            GetCapName = "CAP_JOBCONTROL"
        Case CAP_ALARMS
            GetCapName = "CAP_ALARMS"
        Case CAP_ALARMVOLUME
            GetCapName = "CAP_ALARMVOLUME"
        Case CAP_AUTOMATICCAPTURE
            GetCapName = "CAP_AUTOMATICCAPTURE"
        Case CAP_TIMEBEFOREFIRSTCAPTURE
            GetCapName = "CAP_TIMEBEFOREFIRSTCAPTURE"
        Case CAP_TIMEBETWEENCAPTURES
            GetCapName = "CAP_TIMEBETWEENCAPTURES"
        Case CAP_CLEARBUFFERS
            GetCapName = "CAP_CLEARBUFFERS"
        Case CAP_MAXBATCHBUFFERS
            GetCapName = "CAP_MAXBATCHBUFFERS"
        Case CAP_DEVICETIMEDATE
            GetCapName = "CAP_DEVICETIMEDATE"
        Case CAP_POWERSUPPLY
            GetCapName = "CAP_POWERSUPPLY"
        Case CAP_CAMERAPREVIEWUI
            GetCapName = "CAP_CAMERAPREVIEWUI"
        Case CAP_DEVICEEVENT
            GetCapName = "CAP_DEVICEEVENT"
        Case CAP_PAGEMULTIPLEACQUIRE
            GetCapName = "CAP_PAGEMULTIPLEACQUIRE"
        Case CAP_SERIALNUMBER
            GetCapName = "CAP_SERIALNUMBER"
        Case CAP_FILESYSTEM
            GetCapName = "CAP_FILESYSTEM"
        Case CAP_PRINTER
            GetCapName = "CAP_PRINTER"
        Case CAP_PRINTERENABLED
            GetCapName = "CAP_PRINTERENABLED"
        Case CAP_PRINTERINDEX
            GetCapName = "CAP_PRINTERINDEX"
        Case CAP_PRINTERMODE
            GetCapName = "CAP_PRINTERMODE"
        Case CAP_PRINTERSTRING
            GetCapName = "CAP_PRINTERSTRING"
        Case CAP_PRINTERSUFFIX
            GetCapName = "CAP_PRINTERSUFFIX"
        Case CAP_LANGUAGE
            GetCapName = "CAP_LANGUAGE"
        Case CAP_FEEDERALIGNMENT
            GetCapName = "CAP_FEEDERALIGNMENT"
        Case CAP_FEEDERORDER
            GetCapName = "CAP_FEEDERORDER"
        Case CAP_PAPERBINDING
            GetCapName = "CAP_PAPERBINDING"
        Case CAP_REACQUIREALLOWED
            GetCapName = "CAP_REACQUIREALLOWED"
        Case CAP_PASSTHRU
            GetCapName = "CAP_PASSTHRU"
        Case CAP_BATTERYMINUTES
            GetCapName = "CAP_BATTERYMINUTES"
        Case CAP_BATTERYPERCENTAGE
            GetCapName = "CAP_BATTERYPERCENTAGE"
        Case CAP_POWERDOWNTIME
            GetCapName = "CAP_POWERDOWNTIME"
        Case ICAP_AUTOBRIGHT
            GetCapName = "ICAP_AUTOBRIGHT"
        Case ICAP_BRIGHTNESS
            GetCapName = "ICAP_BRIGHTNESS"
        Case ICAP_CONTRAST
            GetCapName = "ICAP_CONTRAST"
        Case ICAP_CUSTHALFTONE
            GetCapName = "ICAP_CUSTHALFTONE"
        Case ICAP_EXPOSURETIME
            GetCapName = "ICAP_EXPOSURETIME"
        Case ICAP_FILTER
            GetCapName = "ICAP_FILTER"
        Case ICAP_FLASHUSED
            GetCapName = "ICAP_FLASHUSED"
        Case ICAP_GAMMA
            GetCapName = "ICAP_GAMMA"
        Case ICAP_HALFTONES
            GetCapName = "ICAP_HALFTONES"
        Case ICAP_HIGHLIGHT
            GetCapName = "ICAP_HIGHLIGHT"
        Case ICAP_IMAGEFILEFORMAT
            GetCapName = "ICAP_IMAGEFILEFORMAT"
        Case ICAP_LAMPSTATE
            GetCapName = "ICAP_LAMPSTATE"
        Case ICAP_LIGHTSOURCE
            GetCapName = "ICAP_LIGHTSOURCE"
        Case ICAP_ORIENTATION
            GetCapName = "ICAP_ORIENTATION"
        Case ICAP_PHYSICALWIDTH
            GetCapName = "ICAP_PHYSICALWIDTH"
        Case ICAP_PHYSICALHEIGHT
            GetCapName = "ICAP_PHYSICALHEIGHT"
        Case ICAP_SHADOW
            GetCapName = "ICAP_SHADOW"
        Case ICAP_FRAMES
            GetCapName = "ICAP_FRAMES"
        Case ICAP_XNATIVERESOLUTION
            GetCapName = "ICAP_XNATIVERESOLUTION"
        Case ICAP_YNATIVERESOLUTION
            GetCapName = "ICAP_YNATIVERESOLUTION"
        Case ICAP_XRESOLUTION
            GetCapName = "ICAP_XRESOLUTION"
        Case ICAP_YRESOLUTION
            GetCapName = "ICAP_YRESOLUTION"
        Case ICAP_MAXFRAMES
            GetCapName = "ICAP_MAXFRAMES"
        Case ICAP_TILES
            GetCapName = "ICAP_TILES"
        Case ICAP_BITORDER
            GetCapName = "ICAP_BITORDER"
        Case ICAP_CCITTKFACTOR
            GetCapName = "ICAP_CCITTKFACTOR"
        Case ICAP_LIGHTPATH
            GetCapName = "ICAP_LIGHTPATH"
        Case ICAP_PIXELFLAVOR
            GetCapName = "ICAP_PIXELFLAVOR"
        Case ICAP_PLANARCHUNKY
            GetCapName = "ICAP_PLANARCHUNKY"
        Case ICAP_ROTATION
            GetCapName = "ICAP_ROTATION"
        Case ICAP_SUPPORTEDSIZES
            GetCapName = "ICAP_SUPPORTEDSIZES"
        Case ICAP_THRESHOLD
            GetCapName = "ICAP_THRESHOLD"
        Case ICAP_XSCALING
            GetCapName = "ICAP_XSCALING"
        Case ICAP_YSCALING
            GetCapName = "ICAP_YSCALING"
        Case ICAP_BITORDERCODES
            GetCapName = "ICAP_BITORDERCODES"
        Case ICAP_PIXELFLAVORCODES
            GetCapName = "ICAP_PIXELFLAVORCODES"
        Case ICAP_JPEGPIXELTYPE
            GetCapName = "ICAP_JPEGPIXELTYPE"
        Case ICAP_TIMEFILL
            GetCapName = "ICAP_TIMEFILL"
        Case ICAP_BITDEPTH
            GetCapName = "ICAP_BITDEPTH"
        Case ICAP_BITDEPTHREDUCTION
            GetCapName = "ICAP_BITDEPTHREDUCTION"
        Case ICAP_UNDEFINEDIMAGESIZE
            GetCapName = "ICAP_UNDEFINEDIMAGESIZE"
        Case ICAP_IMAGEDATASET
            GetCapName = "ICAP_IMAGEDATASET"
        Case ICAP_EXTIMAGEINFO
            GetCapName = "ICAP_EXTIMAGEINFO"
        Case ICAP_MINIMUMHEIGHT
            GetCapName = "ICAP_MINIMUMHEIGHT"
        Case ICAP_MINIMUMWIDTH
            GetCapName = "ICAP_MINIMUMWIDTH"
        Case ICAP_AUTODISCARDBLANKPAGES
            GetCapName = "ICAP_AUTODISCARDBLANKPAGES"
        Case ICAP_FLIPROTATION
            GetCapName = "ICAP_FLIPROTATION"
        Case ICAP_BARCODEDETECTIONENABLED
            GetCapName = "ICAP_BARCODEDETECTIONENABLED"
        Case ICAP_SUPPORTEDBARCODETYPES
            GetCapName = "ICAP_SUPPORTEDBARCODETYPES"
        Case ICAP_BARCODEMAXSEARCHPRIORITIES
            GetCapName = "ICAP_BARCODEMAXSEARCHPRIORITIES"
        Case ICAP_BARCODESEARCHPRIORITIES
            GetCapName = "ICAP_BARCODESEARCHPRIORITIES"
        Case ICAP_BARCODESEARCHMODE
            GetCapName = "ICAP_BARCODESEARCHMODE"
        Case ICAP_BARCODEMAXRETRIES
            GetCapName = "ICAP_BARCODEMAXRETRIES"
        Case ICAP_BARCODETIMEOUT
            GetCapName = "ICAP_BARCODETIMEOUT"
        Case ICAP_ZOOMFACTOR
            GetCapName = "ICAP_ZOOMFACTOR"
        Case ICAP_PATCHCODEDETECTIONENABLED
            GetCapName = "ICAP_PATCHCODEDETECTIONENABLED"
        Case ICAP_SUPPORTEDPATCHCODETYPES
            GetCapName = "ICAP_SUPPORTEDPATCHCODETYPES"
        Case ICAP_PATCHCODEMAXSEARCHPRIORITIES
            GetCapName = "ICAP_PATCHCODEMAXSEARCHPRIORITIES"
        Case ICAP_PATCHCODESEARCHPRIORITIES
            GetCapName = "ICAP_PATCHCODESEARCHPRIORITIES"
        Case ICAP_PATCHCODESEARCHMODE
            GetCapName = "ICAP_PATCHCODESEARCHMODE"
        Case ICAP_PATCHCODEMAXRETRIES
            GetCapName = "ICAP_PATCHCODEMAXRETRIES"
        Case ICAP_PATCHCODETIMEOUT
            GetCapName = "ICAP_PATCHCODETIMEOUT"
        Case ICAP_FLASHUSED2
            GetCapName = "ICAP_FLASHUSED2"
        Case ICAP_IMAGEFILTER
            GetCapName = "ICAP_IMAGEFILTER"
        Case ICAP_NOISEFILTER
            GetCapName = "ICAP_NOISEFILTER"
        Case ICAP_OVERSCAN
            GetCapName = "ICAP_OVERSCAN"
        Case ICAP_AUTOMATICBORDERDETECTION
            GetCapName = "ICAP_AUTOMATICBORDERDETECTION"
        Case ICAP_AUTOMATICDESKEW
            GetCapName = "ICAP_AUTOMATICDESKEW"
        Case ICAP_AUTOMATICROTATE
            GetCapName = "ICAP_AUTOMATICROTATE"
        Case ACAP_AUDIOFILEFORMAT
            GetCapName = "ACAP_AUDIOFILEFORMAT"
        Case ACAP_XFERMECH
            GetCapName = "ACAP_XFERMECH"
        Case Else
            GetCapName = ""
    End Select
End Function
Public Function GetFileFormatName(iItemType As Integer) As String
    ' Retourne le nom du format de fichier correspondant
    Select Case iItemType
        Case TWFF_TIFF            '/* Tagged Image File Format     */
            GetFileFormatName = "tiff"
        Case TWFF_PICT            '/* Macintosh PICT               */
            GetFileFormatName = "pict"
        Case TWFF_BMP             '/* Windows Bitmap               */
            GetFileFormatName = "bmp"
        Case TWFF_XBM             '/* X-Windows Bitmap             */
            GetFileFormatName = "xbmp"
        Case TWFF_JFIF            '/* JPEG File Interchange Format */
            GetFileFormatName = "jfif"
        Case TWFF_FPX             '/* Flash Pix                    */
            GetFileFormatName = "fpx"
        Case TWFF_TIFFMULTI       '/* Multi-page tiff file         */
            GetFileFormatName = "tiff(multi)"
        Case TWFF_PNG             '/* Portable Network Graphics    */
            GetFileFormatName = "png"
        Case TWFF_SPIFF
            GetFileFormatName = "spiff"
        Case TWFF_EXIF
            GetFileFormatName = "exif"
    End Select
End Function


Public Function GetChunkyName(i) As String
    ' Retourne chunky ou planar
    Select Case i
        Case TWPC_CHUNKY
            GetChunkyName = "chunky"
        Case TWPC_PLANAR
            GetChunkyName = "planar"
    End Select
End Function

Public Function GetPixelFlavorName(i) As String
    ' Retourne la saveur du pixel correspondante
    Select Case i
        Case TWPF_CHOCOLATE             '/* zero pixel represents darkest shade  */
            GetPixelFlavorName = "chocolate"
        Case TWPF_VANILLA               '/* zero pixel represents lightest shade */
            GetPixelFlavorName = "vanilla"
    End Select
End Function

Public Function GetPixelTypeName(i) As String
    ' Retourne la methode de gestion des couleurs (type de point) correspondante
    Select Case i
        Case TWPT_BW    '/* Black and White */
            GetPixelTypeName = "BW"
        Case TWPT_GRAY
            GetPixelTypeName = "GRAY"
        Case TWPT_RGB
            GetPixelTypeName = "RGB"
        Case TWPT_PALETTE
            GetPixelTypeName = "PALETTE"
        Case TWPT_CMY
            GetPixelTypeName = "CMY"
        Case TWPT_CMYK
            GetPixelTypeName = "CMYK"
        Case TWPT_YUV
            GetPixelTypeName = "YUV"
        Case TWPT_YUVK
            GetPixelTypeName = "YUVK"
        Case TWPT_CIEXYZ
            GetPixelTypeName = "CIEXYZ"
    End Select
End Function

Public Function GetSupSizeName(i) As String
    Select Case i
        Case TWSS_NONE
            GetSupSizeName = "none"
        Case TWSS_A4LETTER
            GetSupSizeName = "a4letter"
        Case TWSS_B5LETTER
            GetSupSizeName = "b5letter"
        Case TWSS_USLETTER
            GetSupSizeName = "usletter"
        Case TWSS_USLEGAL
            GetSupSizeName = "uslegal"
' /* Added 1.5 */
        Case TWSS_A5
            GetSupSizeName = "a5"
        Case TWSS_B4
            GetSupSizeName = "b4"
        Case TWSS_B6
            GetSupSizeName = "b6"
        Case TWSS_B
            GetSupSizeName = "b"
' /* Added 1.7 */
        Case TWSS_USLEDGER
            GetSupSizeName = "usledger"
        Case TWSS_USEXECUTIVE
            GetSupSizeName = "usexecutive"
        Case TWSS_A3
            GetSupSizeName = "a3"
        Case TWSS_B3
            GetSupSizeName = "b3"
        Case TWSS_A6
            GetSupSizeName = "a6"
        Case TWSS_C4
            GetSupSizeName = "c4"
        Case TWSS_C5
            GetSupSizeName = "c5"
        Case TWSS_C6
            GetSupSizeName = "c6"
' /* Added 1.8 */
        Case TWSS_4A0
            GetSupSizeName = "4a0"
        Case TWSS_2A0
            GetSupSizeName = "2a0"
        Case TWSS_A0
            GetSupSizeName = "a0"
        Case TWSS_A1
            GetSupSizeName = "a1"
        Case TWSS_A2
            GetSupSizeName = "a2"
        Case TWSS_A7
            GetSupSizeName = "a7"
        Case TWSS_A8
            GetSupSizeName = "a8"
        Case TWSS_A9
            GetSupSizeName = "a9"
        Case TWSS_A10
            GetSupSizeName = "a10"
        Case TWSS_ISOB0
            GetSupSizeName = "isob0"
        Case TWSS_ISOB1
            GetSupSizeName = "isob1"
        Case TWSS_ISOB2
            GetSupSizeName = "isob2"
        Case TWSS_ISOB5
            GetSupSizeName = "isob5"
        Case TWSS_ISOB7
            GetSupSizeName = "isob7"
        Case TWSS_ISOB8
            GetSupSizeName = "isob8"
        Case TWSS_ISOB9
            GetSupSizeName = "isob9"
        Case TWSS_ISOB10
            GetSupSizeName = "isob10"
        Case TWSS_JISB0
            GetSupSizeName = "jisb0"
        Case TWSS_JISB1
            GetSupSizeName = "jisb1"
        Case TWSS_JISB2
            GetSupSizeName = "jisb2"
        Case TWSS_JISB3
            GetSupSizeName = "jisb3"
        Case TWSS_JISB4
            GetSupSizeName = "jisb4"
        Case TWSS_JISB6
            GetSupSizeName = "jisb6"
        Case TWSS_JISB7
            GetSupSizeName = "jisb7"
        Case TWSS_JISB8
            GetSupSizeName = "jisb8"
        Case TWSS_JISB9
            GetSupSizeName = "jisb9"
        Case TWSS_JISB10
            GetSupSizeName = "jisb10"
        Case TWSS_C0
            GetSupSizeName = "c0"
        Case TWSS_C1
            GetSupSizeName = "c1"
        Case TWSS_C2
            GetSupSizeName = "c2"
        Case TWSS_C3
            GetSupSizeName = "c3"
        Case TWSS_C7
            GetSupSizeName = "c7"
        Case TWSS_C8
            GetSupSizeName = "c8"
        Case TWSS_C9
            GetSupSizeName = "c9"
        Case TWSS_C10
            GetSupSizeName = "c10"
        Case TWSS_USSTATEMENT
            GetSupSizeName = "usstatement"
        Case TWSS_BUSINESSCARD
            GetSupSizeName = "businesscard"
    End Select
End Function

Public Function GetSetupXferName(i As Integer) As String
    ' Retourne le type de transfert correspondant
    
    Select Case i
        Case TWSX_NATIVE
            GetSetupXferName = "native"
        Case TWSX_FILE
            GetSetupXferName = "file"
        Case TWSX_MEMORY
            GetSetupXferName = "memory"
    End Select
End Function

Public Function GetUnitsName(i) As String
    ' Retourne le nom de l'unit correspondante
    
    Select Case i
        Case TWUN_INCHES
            GetUnitsName = "inches"
        Case TWUN_CENTIMETERS
            GetUnitsName = "centimeters"
        Case TWUN_PICAS
            GetUnitsName = "picas"
        Case TWUN_POINTS
            GetUnitsName = "points"
        Case TWUN_TWIPS
            GetUnitsName = "twips"
        Case TWUN_PIXELS
            GetUnitsName = "pixels"
    End Select
End Function
   
Public Function GetTypeName(iItemType) As String
    ' Retourne le nom de type correspondant
    
    Select Case iItemType
        Case TW_TYPES.TWTY_INT8
            GetTypeName = "TWTY_INT8"
        Case TW_TYPES.TWTY_INT16
            GetTypeName = "TWTY_INT16"
        Case TW_TYPES.TWTY_INT32
            GetTypeName = "TWTY_INT32"
        Case TW_TYPES.TWTY_UINT8
            GetTypeName = "TWTY_UINT8"
        Case TW_TYPES.TWTY_UINT16
            GetTypeName = "TWTY_UINT16"
        Case TW_TYPES.TWTY_UINT32
            GetTypeName = "TWTY_UINT32"
        Case TW_TYPES.TWTY_BOOL
            GetTypeName = "TWTY_FIX32"
        Case TW_TYPES.TWTY_FIX32
            GetTypeName = "TWTY_FIX32"
        Case TW_TYPES.TWTY_FRAME
            GetTypeName = "TWTY_FRAME"
        Case TW_TYPES.TWTY_STR128
            GetTypeName = "TWTY_STR128"
        Case TW_TYPES.TWTY_STR255
            GetTypeName = "TWTY_STR255"
        Case TW_TYPES.TWTY_STR32
            GetTypeName = "TWTY_STR32"
        Case TW_TYPES.TWTY_STR64
            GetTypeName = "TWTY_STR64"
        Case Else
            GetTypeName = ""
    End Select
End Function
Public Function GetContainerName(l) As String
    ' Retourne le nom de la constante d'un conteneur
    Select Case l
        Case TWON_ARRAY              '/* indicates TW_ARRAY container=  */
            GetContainerName = "TWON_ARRAY"
        Case TWON_ENUMERATION        '/* indicates TW_ENUMERATION container */
            GetContainerName = "TWON_ENUMERATION"
        Case TWON_ONEVALUE           '/* indicates TW_ONEVALUE container    */
            GetContainerName = "TWON_ONEVALUE"
        Case TWON_RANGE              '/* indicates TW_RANGE container=  */
            GetContainerName = "TWON_RANGE"
        Case TWON_ICONID             '/* res Id of icon used in USERSELECT lbox */
            GetContainerName = "TWON_ICONID"
        Case TWON_DSMID              '/* res Id of the DSM version num resource */
            GetContainerName = "TWON_DSMID"
        Case TWON_DSMCODEID          '/* res Id of the Mac SM Code resource=*/
            GetContainerName = "TWON_DSMCODEID"
        Case TWON_DONTCARE8
            GetContainerName = "TWON_DONTCARE8"
        Case TWON_DONTCARE16
            GetContainerName = "TWON_DONTCARE16"
        Case TWON_DONTCARE32
            GetContainerName = "TWON_DONTCARE32"
    End Select
End Function

' ===== RETREIVING TWAIN SPECIAL ANSWERS ==========================================
Public Function FTW_ItemSz(intItemType As Integer) As Integer
    ' Renvoi la taille en bytes d'un type particulier
    
    Select Case intItemType
        Case 0
            FTW_ItemSz = 1
        Case 1
            FTW_ItemSz = 2
        Case 3
            FTW_ItemSz = 1
        Case 4
            FTW_ItemSz = 2
        Case 6
            FTW_ItemSz = 2
        Case 8
            FTW_ItemSz = 16
        Case 9
            FTW_ItemSz = 32
        Case &HA
            FTW_ItemSz = 64
        Case &HB
            FTW_ItemSz = 128
        Case &HC
            FTW_ItemSz = 255
        Case Else
            FTW_ItemSz = 4
    End Select
End Function

' FTW_ReadX() - Lecture d'une variable dans un tableau de bytes
Public Function FTW_ReadUShort(bytes() As Byte, offset As Long) As Long
    ' Lit un integer non sign
    FTW_ReadUShort = bytes(offset) + bytes(offset + 1) * 2 ^ 8
End Function
Public Function FTW_ReadULong(bytes() As Byte, offset As Long) As Double
    ' Lit un long non sign
    
    Dim i   As Integer
    For i = 0 To 3
        FTW_ReadULong = FTW_ReadULong + bytes(offset + i) * 2 ^ (8 * i)
    Next
End Function
Public Function FTW_ReadByte(bytes() As Byte, offset As Long) As Byte
    ' Lit un byte sign
    
    Dim us As Integer
    us = bytes(offset)
    If (us And &H80) Then
        us = -(2 ^ 7) + (us And &H7F)
    End If
    FTW_ReadByte = us
End Function
Public Function FTW_ReadShort(bytes() As Byte, offset As Long) As Integer
    ' Lit un integer sign
    Dim us As Long
    us = bytes(offset) + bytes(offset + 1) * 2 ^ 8
    If (us And &H8000) Then
        us = -(2 ^ 15) + (us And &H7FFF)
    End If
    FTW_ReadShort = CInt(us)
End Function
Public Function FTW_ReadLong(bytes() As Byte, offset As Long) As Long
    ' Lit un entier long sign
        
    Dim ul As Double
    Dim i   As Integer
    For i = 0 To 3
        ul = ul + bytes(offset + i) * 2 ^ (8 * i)
    Next
    If (ul And &H800000) Then
        ul = -(2 ^ 31) + (ul And &H7FFFFFFF)
    End If
End Function
Public Function FTW_ReadFix32(bytes() As Byte, offset As Long) As Single
    ' Lit un flottant de type "fix32"
    
    Dim whole As Single
    Dim frac As Single
    Dim tmp As Double
    whole = FTW_ReadShort(bytes, offset)
    frac = FTW_ReadUShort(bytes, offset + 2)
    tmp = Round((whole + frac / 65536) * 1000) / 1000
    FTW_ReadFix32 = tmp
End Function


' FTW_Any2Enum() - Copie une structure TW_CAPABILITY  partir de son handle
'
Public Function FTW_h2Enum(hCap As Long, twenum As FTW_ENUMERATION)
    ' Extrait un conteneur de type enumration
    
    Dim pEnum   As Long
    Dim pEnumC  As Long
    Dim iType   As Integer
    
    Const MAXBYTES = 4096
    
    pEnum = GlobalLock(hCap)
    Dim bytes(MAXBYTES - 1) As Byte
    
    If (pEnum) Then
        'CpyMemRefAdrFromValAdr ftwAny, pEnum, Len(ftwAny)
        pEnumC = pEnum
        RtlMoveMem bytes(0), ByVal pEnumC, 14
        
        Dim sz  As Long
        
        twenum.ItemType = FTW_ReadUShort(bytes, 0)
        twenum.NumItems = FTW_ReadULong(bytes, 2)
        twenum.CurrentIndex = FTW_ReadULong(bytes, 6)
        twenum.DefaultIndex = FTW_ReadULong(bytes, 10)
        
        sz = twenum.NumItems * FTW_ItemSz(twenum.ItemType) + 14
        If sz > (MAXBYTES) Then sz = MAXBYTES
        
        'Debug.Print "*********************************************************"
        'Debug.Print "type=" & GetTypeName(twenum.ItemType) & " items=" & twenum.NumItems & " sz=" & sz & " curindex=" & twenum.CurrentIndex & " defindex=" & twenum.DefaultIndex
        
        RtlMoveMem bytes(0), ByVal pEnumC, sz
        
        Dim l           As Long
        Dim strTrace    As String
        
        For l = 0 To twenum.NumItems - 1
            twenum.ItemList(l) = FTW_ByteTbl2Value(14, bytes(), twenum.ItemType, l)
            strTrace = strTrace & twenum.ItemList(l) & IIf(l < twenum.NumItems - 1, ", ", "")
        Next
                
        'Debug.Print "Values = {" & strTrace & "}"
        'Debug.Print "*********************************************************"
    
        GlobalUnlock hCap
        GlobalFree hCap
    End If
    
    FTW_h2Enum = True
End Function
Public Function FTW_h2OneValue(hCap As Long, two As FTW_ONEVALUE)
    ' Extrait un conteneur de type "one value"
    
    Dim pOV     As Long
    Dim sz      As Long
    
    Const MAX_TWVAR_SZ = 256
    Dim bytes(MAX_TWVAR_SZ - 1) As Byte
    
    pOV = GlobalLock(hCap)
    If (pOV) Then
        RtlMoveMem bytes(0), ByVal pOV, 2
        
        Dim it As Integer
        two.ItemType = FTW_ReadUShort(bytes, 0)
        sz = FTW_ItemSz(two.ItemType)
        
        If sz > 0 Then
            RtlMoveMem bytes(0), ByVal pOV, sz + 2
            two.value = FTW_ByteTbl2Value(2, bytes, two.ItemType, 0)
        End If
    End If
    
    FTW_h2OneValue = True

End Function

Public Function FTW_h2Range(hCap As Long, twrange As FTW_RANGE)
    ' Extrait un conteneur de type fourchette
    
    Dim pRange   As Long
    Dim pRangeC  As Long
    Dim iType   As Integer
    
    pRange = GlobalLock(hCap)
    Dim bytes(21) As Byte
    
    If (pRange) Then
        'CpyMemRefAdrFromValAdr ftwAny, pRange, Len(ftwAny)
        pRangeC = pRange
        RtlMoveMem bytes(0), ByVal pRangeC, 22
        
        Dim sz  As Long
        
        twrange.ItemType = FTW_ReadUShort(bytes, 0)
        twrange.MinValue = FTW_ByteTbl2Value(2, bytes, twrange.ItemType, 0)
        twrange.MaxValue = FTW_ByteTbl2Value(6, bytes, twrange.ItemType, 0)
        twrange.StepSize = FTW_ByteTbl2Value(10, bytes, twrange.ItemType, 0)
        twrange.DefaultValue = FTW_ByteTbl2Value(14, bytes, twrange.ItemType, 0)
        twrange.CurrentValue = FTW_ByteTbl2Value(18, bytes, twrange.ItemType, 0)
        
        GlobalUnlock hCap
        GlobalFree hCap
    End If
    
    FTW_h2Range = True
End Function


' FTW_h2Array() - Copie une structure TW_ARRAY  partir de son handle
'
Public Function FTW_h2Array(hCap As Long, twarray As FTW_ARRAY)
    ' Extrait un conteneur de type array
    
    Dim pEnum   As Long
    Dim pEnumC  As Long
    Dim iType   As Integer
    
    Const MAXBYTES = 4096
    
    pEnum = GlobalLock(hCap)
    Dim bytes(MAXBYTES - 1) As Byte
    
    If (pEnum) Then
        pEnumC = pEnum
        RtlMoveMem bytes(0), ByVal pEnumC, 6
        
        Dim sz  As Long
        
        twarray.ItemType = FTW_ReadUShort(bytes, 0)
        twarray.NumItems = FTW_ReadULong(bytes, 2)
        
        sz = twarray.NumItems * FTW_ItemSz(twarray.ItemType) + 6
        If sz > (MAXBYTES) Then sz = MAXBYTES
        
        'Debug.Print "*********************************************************"
        'Debug.Print "type=" & GetTypeName(twarray.ItemType) & " items=" & twarray.NumItems & " sz=" & sz & " curindex=" & twarray.CurrentIndex & " defindex=" & twarray.DefaultIndex
        
        RtlMoveMem bytes(0), ByVal pEnumC, sz
        
        Dim l           As Long
        Dim strTrace    As String
        
        For l = 0 To twarray.NumItems - 1
            twarray.ItemList(l) = FTW_ByteTbl2Value(14, bytes(), twarray.ItemType, l)
            strTrace = strTrace & twarray.ItemList(l) & IIf(l < twarray.NumItems - 1, ", ", "")
        Next
                
        'Debug.Print "Values = {" & strTrace & "}"
        'Debug.Print "*********************************************************"
    
        GlobalUnlock hCap
        GlobalFree hCap
    End If
    
    FTW_h2Array = True
End Function

Public Function FTW_ByteTbl2Value(iDecal As Integer, bytes() As Byte, iItemType As Integer, iIndex As Long) As Variant
    ' Extrait et convertit une valeur d'un resultat multiple twain
    
    Dim sz      As Integer
    Dim o       As Long
    sz = FTW_ItemSz(iItemType)
    
    o = (sz * iIndex) + iDecal
    
    Select Case iItemType
        Case TW_TYPES.TWTY_INT8
            FTW_ByteTbl2Value = FTW_ReadByte(bytes(), o)
        Case TW_TYPES.TWTY_INT16
            FTW_ByteTbl2Value = FTW_ReadShort(bytes(), o)
        Case TW_TYPES.TWTY_INT32
            FTW_ByteTbl2Value = FTW_ReadLong(bytes(), o)
        
        Case TW_TYPES.TWTY_UINT8
            FTW_ByteTbl2Value = bytes(o)
        Case TW_TYPES.TWTY_UINT16
            FTW_ByteTbl2Value = FTW_ReadUShort(bytes(), o)
        Case TW_TYPES.TWTY_UINT32
            FTW_ByteTbl2Value = FTW_ReadULong(bytes(), o)
        
        Case TW_TYPES.TWTY_BOOL
            FTW_ByteTbl2Value = Not FTW_ReadUShort(bytes(), o) = 0
        
        Case TW_TYPES.TWTY_FIX32
            FTW_ByteTbl2Value = FTW_ReadFix32(bytes(), o)
        
        Case TW_TYPES.TWTY_FRAME
            FTW_ByteTbl2Value = CStr(FTW_ReadFix32(bytes(), o)) & ";"
            FTW_ByteTbl2Value = FTW_ByteTbl2Value & CStr(FTW_ReadFix32(bytes(), o + 4)) & ";"
            FTW_ByteTbl2Value = FTW_ByteTbl2Value & CStr(FTW_ReadFix32(bytes(), o + 8)) & ";"
            FTW_ByteTbl2Value = FTW_ByteTbl2Value & CStr(FTW_ReadFix32(bytes(), o + 12))
        Case TW_TYPES.TWTY_STR128, TW_TYPES.TWTY_STR255, TW_TYPES.TWTY_STR32, TW_TYPES.TWTY_STR64
            ' STRINGS
            Dim s As String
            s = StrConv(bytes(o), vbUnicode)
            FTW_ByteTbl2Value = s
        Case Else
            Debug.Print "FTW_ByteTbl2Value() * Unknown input data type (" & iItemType & ")"
    End Select
End Function
Private Function dump_list(elts, maxi As Long, cap As Long) As String
    Dim i
    For i = 0 To maxi
        dump_list = dump_list & "- " & i & "=" & FTW_CapValueName(cap, elts(i), True)
        If (i < maxi) Then
            dump_list = dump_list & vbCrLf
        End If
    Next
    dump_list = dump_list & vbCrLf & vbCrLf
End Function
Public Function CapabilityTests() As String
    ' Effectue un test de capacit gnral du driver twain selectionn
    
    ' Cet exemple est fournit en guise de dmonstration d'utilisation avance
    ' des commandes FTW_GetMisc et FTW_SetMisc qui ouvrent bien des possibilits
    ' selon le matriel et les pilotes associs.
        
    ' ATTENTION: Cette routine est propose a titre exprimentale.
    '               De plus elle ne fonctionne pas avec tous les pilotes
        
    Dim intFF As Integer
    intFF = FreeFile
    
    FTW_Close
    FTW_Open
    
    Dim caps(2)
    
    caps(0) = TW_CAP.CAP_SUPPORTEDCAPS
    caps(1) = TW_CAP.CAP_SUPPORTEDCAPSEXT
    caps(2) = TW_CAP.CAP_EXTENDEDCAPS
    
    Dim i As Long, conT As Single, hCap As Long
    Dim twacap  As FTW_ARRAY
    Dim twa     As FTW_ARRAY
    Dim twe     As FTW_ENUMERATION
    Dim twr     As FTW_RANGE
    Dim two     As FTW_ONEVALUE
    
    Dim capmem(&H1300) As Boolean
    
    Dim buffer  As String
    
    For i = 0 To 2
        buffer = buffer & "*********************************************************************" & vbCrLf
        buffer = buffer & "** " & GetCapName(CLng(caps(i))) & " (" & i & ")" & vbCrLf
        buffer = buffer & "*********************************************************************" & vbCrLf
        buffer = buffer & "-" & vbCrLf
        hCap = True
        If (FTW_GetMisc(caps(0), conT, hCap)) Then
            If Not FTW_h2Array(hCap, twacap) Or conT <> TW_CONTAINERS.TWON_ARRAY Then Exit Function
            
            Dim j
            Dim strCap As String
            
            For j = 0 To twacap.NumItems - 1
                hCap = True
                Dim cap As Long
                cap = CLng(twacap.ItemList(j))
                
                If (FTW_GetMisc(cap, conT, hCap) And Not capmem(cap)) Then
                    strCap = GetCapName(cap)
                    buffer = buffer & "****" & String(Len(strCap), "*") & "****" & vbCrLf
                    buffer = buffer & "*** " & strCap & " ***" & vbCrLf & vbCrLf
                    
                    capmem(cap) = True
                    buffer = buffer & "  Container Type: " & GetContainerName(conT) & vbCrLf
                    Select Case conT
                        Case TW_CONTAINERS.TWON_ARRAY
                            If Not FTW_h2Array(hCap, twa) Then Exit Function
                            buffer = buffer & "  Data......Type: " & GetTypeName(twa.ItemType) & vbCrLf
                            buffer = buffer & dump_list(twa.ItemList, twa.NumItems - 1, cap) & vbCrLf & vbCrLf
                        Case TW_CONTAINERS.TWON_ENUMERATION
                            If Not FTW_h2Enum(hCap, twe) Then Exit Function
                            buffer = buffer & "  Data......Type: " & GetTypeName(twe.ItemType) & vbCrLf
                            buffer = buffer & "  Current..Index: " & (twe.CurrentIndex) & vbCrLf
                            buffer = buffer & "  Default..Index: " & (twe.DefaultIndex) & vbCrLf & vbCrLf
                            buffer = buffer & dump_list(twe.ItemList, twe.NumItems - 1, cap) & vbCrLf
                        Case TW_CONTAINERS.TWON_ONEVALUE
                            If Not FTW_h2OneValue(hCap, two) Then Exit Function
                            buffer = buffer & "  Data......Type: " & GetTypeName(two.ItemType) & vbCrLf
                            buffer = buffer & "  Current..Value: " & (two.value) & vbCrLf
                            
                        Case TW_CONTAINERS.TWON_RANGE
                            If Not FTW_h2Range(hCap, twr) Then Exit Function
                            buffer = buffer & "  Data      Type: " & GetTypeName(twr.ItemType) & vbCrLf
                            buffer = buffer & "  Current..Value: " & (twr.CurrentValue) & vbCrLf
                            buffer = buffer & "  Default..Value: " & (twr.DefaultValue) & vbCrLf
                            buffer = buffer & "  Min......Value: " & (twr.MinValue) & vbCrLf
                            buffer = buffer & "  Max......Value: " & (twr.MaxValue) & vbCrLf
                            buffer = buffer & "  Step......Size: " & (twr.StepSize) & vbCrLf
                            
                        Case Else
                            buffer = buffer & "Type de container inatendu." & vbCrLf
                    End Select
                    buffer = buffer & vbCrLf & vbCrLf
                    Debug.Print buffer
                    buffer = ""
                    DoEvents
                End If
            Next j
        End If
    Next
    
    CapabilityTests = buffer
End Function


